-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Script:  Auto_LOD
-- Version: 1.11
-- Build:   2015-02-07
-- By:      oe3gsu@x-plane.at
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Description:
-- 1) Set "User  editable values" as described
-- 2) Start X-Plane or restart LUA-Scripts via "Plugins/FlyWithLUA/Reload all LUA Scripts"
-- 3) Assign a button or key in the X-Plane-Menu to switch auto/manual-mode and to change the LOD-value
-- 4) You can change the LOD-value with positioning your mouse-pointer in the "LOD"-Field an uns your scroll-wheel
-- 5) To change from auto- to manual-mode click into the FPS-field (red background in auto-mode)
-- 6) have fun
-- 
-- Hints:
-- Flying fast and high: use lower values for "LOD_fps_min" (> 20) and "LOD_fps_max" (e.g. 30)
-- Flying low and slow (e.g. Heli): use higher values for "LOD_fps_min" (e.g. 30) and "LOD_fps_max" (e.g. 40)
-- but remember: your actual fps depends on the installed addons, sceneries, and so on.
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

-- USER editable values -------------------------------------------------------------------------------

local LOD_fps_min = 30     -- minimum FPS, if FPS drops below this value, LOD will be reduced
local LOD_fps_max = 40     -- maximum FPS, if FPS raises above this value, LOD will be increased. must be greater than LOD_fps_min + 10
local LOD_fps_time = 500   -- timeout [msec]: time to wait before to set new LOD (100 .. 1000 recomended) 

local LOD_posX = 10        -- horziontal position of window 
local LOD_posY = 10        -- vertikal  position of window 

LOD_Mode_auto = true       -- Start in Auto-Mode set to "true" or "false"
LOD_Sonderfunktion = true  -- Special function (Landing Lights switch "auto-mode")
LOD_Sonder_value_off = 1.0 -- Value if landing-light goes off (manual mode)

---------- DONT edit anything below this line !!! -----------------------------------------------------

-- STARTUP --------------------------------------------------------------------------

require("graphics")

DataRef("LOD_xp_lod_akt", "sim/private/controls/reno/LOD_bias_rat", "writable")
DataRef("LOD_xp_framerate", "sim/operation/misc/frame_rate_period", "readonly")   -- 1/frame_rate_period = FPS
DataRef("LOD_xp_aif_akt", "sim/private/controls/reno/aniso_filter", "writable")
DataRef("LOD_xp_llight", "sim/cockpit/electrical/landing_lights_on", "readonly")
DataRef("LOD_xp_obf_akt", "sim/private/controls/tex/ortho_boost_factor", "writable")

DataRef("LOD_fsaa_x", "sim/private/controls/hdr/fsaa_ratio_x", "writable")
DataRef("LOD_fsaa_y", "sim/private/controls/hdr/fsaa_ratio_y", "writable")
DataRef("LOD_post_aa", "sim/private/controls/hdr/use_post_aa", "writable")

DataRef("LOD_xp_paused", "sim/time/paused", "readonly")

local LOD_version = "Auto_LOD 1.3"
local LOD_akt_time = os.clock()
local LOD_akt_mode = 0
local LOD_old_llight = 0
local LOD_hdr_wert = 0
local LOD_hdr_strg = ""
local LOD_fps_average = 0

if LOD_fps_min <= 20 then LOD_fps_min = 20 end                                    -- set min-value
if LOD_fps_max <= LOD_fps_min + 10 then LOD_fps_max = LOD_fps_min + 10 end        -- set max-value

  local count = 0


if LOD_fsaa_x == 1 and LOD_fsaa_y == 1 and LOD_post_aa == 2 then       -- FXAA
  LOD_hdr_wert = 1
elseif LOD_fsaa_x == 1 and LOD_fsaa_y == 2 and LOD_post_aa == 2 then   -- 2xSSAA+FXAA
  LOD_hdr_wert = 2
elseif LOD_fsaa_x == 2 and LOD_fsaa_y == 2 and LOD_post_aa == 0 then   -- 4xSSAA
  LOD_hdr_wert = 3
elseif LOD_fsaa_x == 2 and LOD_fsaa_y == 2 and LOD_post_aa == 2 then   -- 4xSSAA+FXAA
  LOD_hdr_wert = 4
elseif LOD_fsaa_x == 2 and LOD_fsaa_y == 4 and LOD_post_aa == 2 then   -- 8xSSAA+FXAA
  LOD_hdr_wert = 5
else
  LOD_hdr_wert=0                                                            -- none
  LOD_fsaa_x = 1
  LOD_fsaa_y = 1
  LOD_post_aa = 0
 end

function LOD_show_display() --------------------------------------------------------SHOW Display
  XPLMSetGraphicsState(0,0,0,1,1,0,0)
  
  graphics.set_color(0.5, 0.5, 0.5, 0.5)
  graphics.draw_rectangle(LOD_posX, LOD_posY, LOD_posX + 60, LOD_posY + 15)
  
  if LOD_Mode_auto == false then
    graphics.set_color(0.5, 0.5, 0.5, 0.5)
    graphics.draw_rectangle(LOD_posX, LOD_posY+15, LOD_posX + 60, LOD_posY + 30)
  else
    graphics.set_color(1.0, 0.0, 0.0, 0.5)
    graphics.draw_rectangle(LOD_posX, LOD_posY+15, LOD_posX + 60, LOD_posY + 30)
    
  end
  graphics.set_color(1.0, 1.0, 1.0, 1.0)
  draw_string_Helvetica_10(LOD_posX - 3 , LOD_posY + 33, LOD_version) -- .. " | " .. LOD_fps_average .. " | " .. count)
  graphics.draw_line(LOD_posX, LOD_posY, LOD_posX, LOD_posY+30)       -- left 
  graphics.draw_line(LOD_posX, LOD_posY+30, LOD_posX+60, LOD_posY+30) -- top 
  graphics.draw_line(LOD_posX+60, LOD_posY+30, LOD_posX+60, LOD_posY) -- right 
  graphics.draw_line(LOD_posX, LOD_posY, LOD_posX+60, LOD_posY)       -- bottom 
  graphics.draw_line(LOD_posX, LOD_posY+15, LOD_posX+60, LOD_posY+15)       -- middle 
  
  if 1/LOD_xp_framerate < 21 then graphics.set_color(1.0, 0.0, 0.0, 1.0) else graphics.set_color(1.0, 1.0, 1.0, 1.0) end
  draw_string_Helvetica_12(LOD_posX + 3 , LOD_posY + 17, "FPS: " .. string.format("%2.0f", 1/LOD_xp_framerate)) -- LOD_xp_framerate
  
  if LOD_akt_mode == 0 then
    draw_string_Helvetica_12(LOD_posX + 3 , LOD_posY + 2, "LOD: " .. string.format("%1.1f", LOD_xp_lod_akt))
  elseif LOD_akt_mode == 1 then
    if LOD_hdr_wert == 0 then LOD_hdr_strg = "none" end
    if LOD_hdr_wert == 1 then LOD_hdr_strg = "FXAA" end
    if LOD_hdr_wert == 2 then LOD_hdr_strg = "2S+F" end
    if LOD_hdr_wert == 3 then LOD_hdr_strg = " 4F " end
    if LOD_hdr_wert == 4 then LOD_hdr_strg = "4S+F" end
    if LOD_hdr_wert == 5 then LOD_hdr_strg = "8S+F" end
    draw_string_Helvetica_12(LOD_posX + 3 , LOD_posY + 2, "H: " .. LOD_hdr_strg)
  elseif LOD_akt_mode == 2 then
    draw_string_Helvetica_12(LOD_posX + 3 , LOD_posY + 2, "AIF:  " .. string.format("%2.0f", LOD_xp_aif_akt))
  elseif LOD_akt_mode == 3 then
    draw_string_Helvetica_12(LOD_posX + 3 , LOD_posY + 2, "OBF: " .. string.format("%1.1f", LOD_xp_obf_akt))
  end
  
end
do_every_draw("LOD_show_display()")

function LOD_main() -------------------------------------------------------------------------------------------------Auto-Mode
  if LOD_Sonderfunktion == true then 
    if LOD_xp_llight == 1 and LOD_old_llight == 0 then 
      LOD_Mode_auto = true
    end
    if LOD_xp_llight == 0 and LOD_old_llight == 1 then 
      LOD_Mode_auto = false
      LOD_xp_lod_akt = LOD_Sonder_value_off
    end
    LOD_old_llight = LOD_xp_llight
  end
  
  if LOD_Mode_auto == true and LOD_xp_paused == 0 then                                              -- do it only when in "auto"-mode
    
    if os.clock() > LOD_akt_time + (LOD_fps_time/1000) then                  -- check if wait-time is over
      local akt_fps = 1/LOD_xp_framerate
      if akt_fps > LOD_fps_max then 
        LOD_xp_lod_akt = LOD_xp_lod_akt -0.1 
      end
      if akt_fps < LOD_fps_min then 
        LOD_xp_lod_akt = LOD_xp_lod_akt +0.1 
      end
      if LOD_fps_average > LOD_fps_min + 1 and akt_fps > LOD_fps_min then
        LOD_xp_lod_akt = LOD_xp_lod_akt - ((LOD_fps_average - LOD_fps_min) / 200)
      end
      LOD_akt_time = os.clock()
    end
  end
  if LOD_xp_lod_akt > 9.9 then LOD_xp_lod_akt = 9.9 end
  if LOD_xp_lod_akt < 0 then LOD_xp_lod_akt = 0 end
  
  if LOD_xp_obf_akt > 1.0 then LOD_xp_obf_akt = 1.0 end
  if LOD_xp_obf_akt < 0 then LOD_xp_obf_akt = 0 end
  
    if LOD_hdr_wert == 0 then
      LOD_fsaa_x = 1
      LOD_fsaa_y = 1
      LOD_post_aa = 0
    end
    if LOD_hdr_wert == 1 then
      LOD_fsaa_x = 1
      LOD_fsaa_y = 1
      LOD_post_aa = 2
    end
    if LOD_hdr_wert == 2 then
      LOD_fsaa_x = 1
      LOD_fsaa_y = 2
      LOD_post_aa = 2
    end
    if LOD_hdr_wert == 3 then
      LOD_fsaa_x = 2
      LOD_fsaa_y = 2
      LOD_post_aa = 0
    end
    if LOD_hdr_wert == 4 then
      LOD_fsaa_x = 2
      LOD_fsaa_y = 2
      LOD_post_aa = 2
    end
    if LOD_hdr_wert == 5 then
      LOD_fsaa_x = 2
      LOD_fsaa_y = 4
      LOD_post_aa = 2
    end
  
  
end

do_every_frame("LOD_main()")

function LOD_calc_average()
  --local average = 0
  --local count = 0
  if LOD_xp_paused == 0 then 
    LOD_fps_average = ((LOD_fps_average * count) + (1/LOD_xp_framerate)) / (count + 1)
    count = count + 1
    if count > 5 then count = 5 end
  end
-- LOD_fps_average
end

do_often("LOD_calc_average()")

function LOD_mouse_wheel_events()  --------------------------------------------------------------------------------- Mouse Wheel -----------
  if MOUSE_X > LOD_posX and MOUSE_X < LOD_posX+60 and MOUSE_Y > LOD_posY and MOUSE_Y < LOD_posY+15 then                  -- in LOD-Field
  if LOD_akt_mode == 0 then                                                                                            -- change LOD
      LOD_xp_lod_akt = LOD_xp_lod_akt - (MOUSE_WHEEL_CLICKS / 10)
    elseif LOD_akt_mode == 1 then
      if MOUSE_WHEEL_CLICKS > 0 then
        LOD_hdr_wert = LOD_hdr_wert + 1
        if LOD_hdr_wert > 5 then LOD_hdr_wert = 5 end
      else
        LOD_hdr_wert = LOD_hdr_wert - 1
        if LOD_hdr_wert < 0 then LOD_hdr_wert = 0 end
      end
      
    elseif LOD_akt_mode == 2 then                                                                                        -- change Anisotropic filter
      if MOUSE_WHEEL_CLICKS > 0 then
        LOD_xp_aif_akt = LOD_xp_aif_akt * 2
        if LOD_xp_aif_akt > 16 then LOD_xp_aif_akt = 16 end
      else
        LOD_xp_aif_akt = LOD_xp_aif_akt / 2
        if LOD_xp_aif_akt < 1 then LOD_xp_aif_akt = 1 end
      end
    elseif LOD_akt_mode == 3 then                                                                                            -- change LOD
      LOD_xp_obf_akt = LOD_xp_obf_akt - (MOUSE_WHEEL_CLICKS / 10)
   end
    RESUME_MOUSE_WHEEL = true
  end
end

do_on_mouse_wheel("LOD_mouse_wheel_events()")

function LOD_mouse_click_events() --------------------------------------------------------------------------------- Mouse Click -----------
  if MOUSE_STATUS == "down" and MOUSE_X > LOD_posX and MOUSE_X < LOD_posX+60 and MOUSE_Y > LOD_posY+15 and MOUSE_Y < LOD_posY+30 then --------------Click in FPS-Field
    LOD_Mode_auto = not LOD_Mode_auto
  end
  
  if MOUSE_STATUS == "down" and MOUSE_X > LOD_posX and MOUSE_X < LOD_posX+60 and MOUSE_Y > LOD_posY and MOUSE_Y < LOD_posY+15 then --------------Click in Display-Field
    LOD_akt_mode = LOD_akt_mode + 1
    if LOD_akt_mode == 4 then LOD_akt_mode = 0 end
    
  end
  
end

do_on_mouse_click("LOD_mouse_click_events()")

create_command("FlyWithLua/Auto_LOD/switch_Mode", "Switch auto/manual", "LOD_Mode_auto = not LOD_Mode_auto", "", "")

create_command("FlyWithLua/Auto_LOD/LOD_up", "LOD up", "LOD_xp_lod_akt = LOD_xp_lod_akt +0.1", "", "")

create_command("FlyWithLua/Auto_LOD/LOD_down", "LOD down", "LOD_xp_lod_akt = LOD_xp_lod_akt -0.1", "", "")

















